unit Unit1;

{
  polygon contractor/expander
  july 2023
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, Grids;

const boxwidth  = 961;  //dimensions of paintbox
      boxheight = 801;  //..

type
  TForm1 = class(TForm)
    PaintBox1: TPaintBox;
    msgtext: TStaticText;
    clearbtn: TBitBtn;
    Label2: TLabel;
    drawbtn: TSpeedButton;
    modifyBtn: TSpeedButton;
    Label6: TLabel;
    xposlabel: TLabel;
    Label8: TLabel;
    yposlabel: TLabel;
    pointsgrid: TStringGrid;
    crsr1: TImage;
    crsr2: TImage;
    undoBtn: TBitBtn;
    repaintBtn: TBitBtn;
    Vbox: TPaintBox;
    Hbox: TPaintBox;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    saveBtn: TBitBtn;
    BitBtn1: TBitBtn;
    procedure FormPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure clearbtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure drawbtnClick(Sender: TObject);
    procedure modifyBtnClick(Sender: TObject);
    procedure undoBtnClick(Sender: TObject);
    procedure repaintBtnClick(Sender: TObject);
    procedure HboxPaint(Sender: TObject);
    procedure HboxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure HboxMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure HboxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SpeedButton2Click(Sender: TObject);
    procedure SpeedButton1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SpeedButton1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure saveBtnClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

procedure repaintall;
procedure showpoints;

implementation

{$R *.dfm}

uses poly_unit,clock_unit;

const antivirus = 'xxxxxxxyyyyyyyyyyyyyyzzzzzzzzzzzzz'; //false virus warning

type TmouseAction = (maDown,maMove,maUp);
     TmainBtn = (mbDraw,mbModify);

var map1 : Tbitmap;
    map2 : Tbitmap;
    map1rect : Trect;    //modified area of map1
    map1full : boolean;  //map1 rectangle valid
    map2rect : Trect;
    map2full : boolean;
    paintRect: Trect;

    mouseX   : longInt;   //x mouse coordinate
    mouseY   : longInt;   //y ...
    oldX     : longInt;
    oldY     : longInt;
    mouseAction : Tmouseaction;  //move, down, up
    mcount   : byte = 0;  //drawing control
    selpoint : word;      //modify on this point

    mainbutton : TMainbtn;

    Hbias : smallInt = 480;
    Vbias : smallInt = 400;
    HVmouseDown : boolean; //scrolling
    PEF : boolean;         //process expansion flag

    savedpoints : array[1..maxpoint] of TFpoint;
    savedpcount : byte;


function pixel2X(pix : smallInt) : single;
begin
 result := 0.025*(pix-Hbias);
end;

function pixel2Y(pix : smallInt) : single;
begin
 result := 0.025*(Vbias-pix);
end;

function X2pixel(xx: single): smallInt;
begin
 result := round(40*xx+Hbias);
end;

function Y2pixel(yy : single) : smallInt;
begin
 result := round(Vbias-40*yy);
end;

function coord2string(xy : single) : string;
var i,pp,w : byte;
begin
 result := formatfloat('##0.###',xy);
 pp := pos('.',result);
 if pp = 0 then w := 3-length(result) else w := 4-pp;
 for i := 1 to w do insert(' ',result,1);
end;

function fp2pixel(const fp : TFpoint) : Tpoint;
begin
 result.X := X2pixel(fp.x);
 result.y := Y2pixel(fp.y);
end;

function pixel2fp(const pix : Tpoint) : TFpoint;
begin
 result.x := pixel2X(pix.X);
 result.y := pixel2Y(pix.Y);
end;

function adjustHVbias(x : smallInt) : smallInt;
//round x to multiples of 40
begin
 result := ((x+20) div 40)*40;
end;

//---drawing support---

function packrect(x1,y1,x2,y2 : longint): Trect;
//make rectangle 
begin
 with result do
  begin
   if x2 > x1 then
    begin
     left := x1-1;
     right := x2 + 1;
    end
    else begin
          left := x2-1;
          right := x1+1;
         end;
   if y1 < y2 then
    begin
     top := y1-1;
     bottom := y2 + 1;
    end
    else begin
          top := y2-1;
          bottom := y1+1;
         end;
  end;//with
end;

function unirect(const r1,r2 : Trect) : Trect;
//unite rectangles r1,r2
begin
 with result do
  begin
   if r1.Left < r2.Left then left := r1.left else left := r2.Left;
   if r1.Right > r2.Right then right := r1.Right else right := r2.Right;
   if r1.Top < r2.Top then top := r1.top else top := r2.Top;
   if r1.Bottom > r2.Bottom then bottom := r1.Bottom else bottom := r2.Bottom;
  end;//with
end;

procedure update1(const r : Trect);
//update map1rect
begin
 if map1full then map1rect := unirect(r, map1rect) else map1rect := r;
 map1full := true;
end;

procedure update2(const r : Trect);
//update map2rect
begin
 if map2full then map1rect := unirect(r, map2rect) else map2rect := r;
 map2full := true;
end;

procedure update12(const r : Trect);
//update maps1,2 on draw
begin
 update1(r);
 update2(r);
end;

procedure copymap12;
//copy rectangle from map1 to map2
begin
 if map1full then
  begin
   map2.Canvas.CopyRect(map1rect,map1.Canvas,map1rect);
   map2rect := unirect(map1rect,map2rect);
   map2full := true;
   map1full := false;
  end;
end;

procedure copymap2box;
//copy rectangle from map2 to paintbox1
begin
 if map2full then
  begin
   form1.PaintBox1.Canvas.CopyRect(map2rect,map2.Canvas,map2rect);
   map2full := false;
  end;
end;

function closecheck : boolean;
//return true if closed polygon
begin
 result := (fpoints[1].x = fpoints[pcount].X) and
                  (fpoints[1].y = fpoints[pcount].y);
end;

procedure clearmap1;
//paint grid
var i : word;
begin
 with map1 do with canvas do
  begin
   brush.Color := $f0ffff;
   fillrect(rect(0,0,width,height));
   pen.Color := $fff0f0;
   pen.Width := 1;
   i := 0;
   while (i < boxwidth) do
    begin
     if (i mod 40) = 0 then pen.Color := $ffc0c0 else pen.Color := $ffe0e0;
     moveto(i,0);
     lineto(i,boxheight);
     inc(i,20);
    end;
   i := 0;
   while i < boxheight do
    begin
     if (i mod 40) = 0 then  pen.Color := $ffc0c0 else pen.Color := $ffe0e0;
     moveto(0,i);
     lineto(boxwidth,i);
     inc(i,20);
    end;
   pen.Color := $ff0000;
   i := Vbias*40;
   moveto(0,i);
   lineto(boxwidth,i);
   i := Hbias*40;
   moveto(i,0);
   lineto(i,boxheight);
  end;
 update1(rect(0,0,boxwidth,boxheight));
end;

procedure drawline(bm : Tbitmap; p1,p2 : TPoint; c : longInt);
//paint line in bm from p1 to p2, color = c
//update paintrect
begin
 with bm do with canvas do
  begin
   pen.Width := 2;
   pen.color := c;
   moveto(p1.x,p1.y);
   lineto(p2.x,p2.y);
  end;
 paintrect := packrect(p1.X,p1.Y,p2.X,p2.y);
end;

procedure drawpoint(bm : Tbitmap; p : Tpoint);
//draw a point in map1,2 --> box
//update paintrect
begin
 with paintrect do
  begin
   Left := p.x - 2;
   Right := p.x + 3;
   Top := p.y - 2;
   Bottom := p.y + 3;
  end;
 with bm.Canvas do
  begin
   brush.color := $000000;
   pen.color := $000000;
   fillrect(paintrect);
  end;
end;

procedure drawmodlines(bm : Tbitmap; c : LongInt);
//draw lines to/from selpoint , color = c
var closed : boolean;
    pts1,pts2,pts3 : TPoint;
begin
 if (selpoint > 1) and (selpoint < pcount) then
  begin
   pts1 := fp2pixel(fpoints[selpoint-1]);
   pts2 := fp2pixel(fpoints[selpoint]);
   pts3 := fp2pixel(fpoints[selpoint+1]);
   drawline(bm,pts1,pts2,c);
   update12(paintrect);
   drawline(bm,pts2,pts3,c);
   update12(paintrect);
  end
  else begin
        closed := closecheck;
        if (selpoint = 1) or closed then
         begin
          pts1 := fp2pixel(fpoints[1]);
          pts2 := fp2pixel(fpoints[2]);
          drawline(bm,pts1,pts2,c);
          update12(paintrect);
         end;
        if (selpoint = pcount) or closed then
         begin
          pts1 := fp2pixel(fpoints[pcount-1]);
          pts2 := fp2pixel(fpoints[pcount]);
          drawline(bm,pts1,pts2,c);
          update12(paintrect);
         end;
       end;
end;

procedure repaintmap1;
var i : word;
    p1,p2 : Tpoint;
begin
 clearmap1;
 if savedpcount > 0 then
  begin
   p1 := fp2pixel(savedpoints[1]);
   for i := 2 to savedpcount do
    begin
     p2 := fp2pixel(savedpoints[i]);
     drawline(map1,p1,p2,$808080);
     p1 := p2;
    end;
  end;
  p1 := fp2pixel(fpoints[1]);
  drawpoint(map1,p1);
  for i := 2 to pcount do
   begin
    p2 := fp2pixel(fpoints[i]);
    drawline(map1,p1,p2,$000000);
    drawpoint(map1,p2);
    p1 := p2;
   end;
end;

procedure undo;
//selpoint <> 0 : remove selpoint
//selpoint = 0  : remove last 
//clear, repaint map1
var i : word;
begin
 if pcount = 0 then exit;

 if selpoint = 0 then selpoint := pcount;
 for i := selpoint to pcount-1 do       //shift points down
  begin
   fpoints[i].X := fpoints[i+1].x;
   fpoints[i].Y := fpoints[i+1].y;
   with form1.pointsgrid do
    begin
     cells[1,i-1] := cells[0,i+1];
     cells[2,i-1] := cells[1,i+1];  
    end;//with..
  end;//for..
 fpoints[pcount].X := 0;
 fpoints[pcount].Y := 0;
 with form1.pointsgrid do
  begin
   cells[1,pcount] := '';
   cells[2,pcount] := '';
  end;

 dec(pcount);
 if pcount = 1 then
  begin
   pcount := 0;
   with fpoints[1] do
    begin
     x := 0;
     y := 0;
    end; 
   with form1.pointsgrid do
    begin
     cells[1,1] := '';
     cells[2,1] := '';
    end;
  end;
 repaintmap1;
 copymap12;
 copymap2box;
 form1.msgtext.Caption := '';
end;

procedure clearPointsGrid;
//erases pointsGrid
var i : word;
begin
 with form1.pointsgrid do
  begin
   for i := 1 to maxpoint do
    begin
     cells[1,i] := ''; cells[2,i] := '';
    end;
   col := 1; row := 1; 
  end;//with
end;

procedure showpoints;
//in grid
var i : byte;
begin
 clearpointsgrid;
 for i := 1 to pcount do
  with form1.pointsgrid do
   begin
    cells[1,i] := coord2string(Fpoints[i].x);
    cells[2,i] := coord2string(Fpoints[i].y);
  end;
end;

procedure showcursor;
//erase old cursor, new at mouseX,mouseY
//mcount = 1
var r : TRect;
    bm : TBitmap;
begin
 r.Left := oldX-4;
 r.Right := oldX+6;
 r.Top := oldY-4;
 r.Bottom := oldY+6;
 if selpoint <> 0 then bm := form1.crsr2.picture.bitmap
  else bm := form1.crsr1.picture.bitmap;

 with form1.PaintBox1.Canvas do
  begin
   copyrect(r,map2.Canvas,r);    //erase old cursor
   draw(mouseX-4,mouseY-4,bm);   //paint new cursor
  end;
end;

procedure onPointCheck;
//check cursor on first or on last point
//set selpoint = 0 if not  or on 1st & last point
var dx,dy : smallInt;
    pts : Tpoint;
begin
 selpoint := 0;
 if pcount = 0 then exit;

 pts := fp2pixel(fpoints[pcount]);
 dx := abs(mouseX - pts.x);
 dy := abs(mouseY - pts.y);
 if (dx < 4) and (dy < 4) then
  begin
   selpoint := pcount;
   exit;
  end;
 pts := fp2pixel(fpoints[1]);
 dx := abs(mouseX - pts.x);
 dy := abs(mouseY - pts.y);
 if (dx < 4) and (dy < 4) then
 selpoint := 1;
end;

procedure onAnyPointCheck;
//selpoint 0:not on point; <> 0:on point
var i : word;
    pts1 : Tpoint;
begin
 selpoint := 0;
 for i := 1 to pcount do
  begin
   pts1 := fp2pixel(fpoints[i]);
   if (abs(mouseX - pts1.x) < 4) and
      (abs(mouseY - pts1.Y) < 4) then
    begin
     selpoint := i;
     break;
    end;
  end;
end;

procedure registerpoint(n : word);
//insert (mouseX,mouseY) coordinates at point n
var i : word;
begin
 if pcount > 0 then
  for i := pcount downto n do
   with fpoints[i+1] do
    begin
     x := fpoints[i].X;
     y := fpoints[i].Y;
    end;
 with fpoints[n] do
  begin
   x := pixel2X(mouseX);
   y := pixel2Y(mouseY);
  end;
 inc(pcount);
 with form1.pointsgrid do
  for i := 1 to pcount do
   begin
    cells[1,i] := coord2string(fpoints[i].x);
    cells[2,i] := coord2string(fpoints[i].y);   
   end;
end;

procedure repaintall;
begin
 repaintmap1;
 copymap12;
 copymap2box;
end;

//---main procedures---

procedure drawProc;
//handle mouse events for drawing
//mcount 0 : cursor moving on paintbox
//       1 : after mouse down
//       2 : move after mouse down, drawing
var oldselpoint : word;
    p : TPoint;
    pts : Tpoint;
begin
 p.x := mouseX; p.y := mouseY;
 case mouseAction of
  maMove :
     case mcount of
       0 : begin
            onPointcheck;
           end;
       1 : begin
            mcount := 2;
           end;
       2 : begin
            copymap12;
            pts := fp2pixel(fpoints[selpoint]);
            drawline(map2,pts,p,$000000);
            update12(paintrect);
            copymap2box;
           end;
      end;//case
  maDown :
     case mcount of
       0 : begin
            if pcount = 0 then
             begin
              registerpoint(1);
              selpoint := 1;
              mcount := 1;
              drawpoint(map1,p);
              update1(paintrect);
              copymap12;
              copymap2box;
             end
             else
              if (pcount < maxpoint) then
               begin
                if selpoint <> 0 then mcount := 1;
               end
               else form1.msgtext.Caption := 'puntenlijst is vol';
           end;
       1,2 : ;
     end;//case
  maUp   :
    case mcount of
      0 : selpoint := 0;
      1 : begin
           mcount := 0;
           selpoint := 0;
           if pcount = 1 then undo;
          end;
      2 : begin
           mcount := 0;
           oldselpoint := selpoint;
           onPointCheck;
           if selpoint = oldselpoint then undo  //start = end
            else
             begin
              pts := fp2pixel(fpoints[oldselpoint]);
              drawline(map1,pts,p,$000000);
              update1(paintrect);
              drawpoint(map1,p);
              update1(paintrect);
              copymap12;
              copymap2box;
              if oldselpoint = pcount then inc(oldselpoint);//append
              registerpoint(oldselpoint);
             end;
           if pcount = 1 then
            begin
             undo;
             selpoint := 0;
            end;
           onPointCheck;
          end;
    end;//case
 end;//case
 showcursor;
end;

procedure modifyProc;
//handle mouse events for modification of drawing
//mcount 0 : cursor moving on paintbox
//       1 : after mouse down
//       2 : move after mouse down, modifying
var closed : boolean;
begin
 case mouseAction of
  maMove :
     case mcount of
       0 : begin
            onAnyPointCheck;
           end;
       1 : begin
            mcount := 2;
           end;
       2 : begin
            copymap12;
            closed := closecheck;
            fpoints[selpoint].X := pixel2x(mouseX);
            fpoints[selpoint].Y := pixel2Y(mouseY);
            with form1.pointsgrid do
             begin
              cells[1,selpoint] := coord2string(pixel2X(mouseX));
              cells[2,selpoint] := coord2string(pixel2Y(mouseY));
             end;
            if (selpoint = 1) and closed then
             begin
              fpoints[pcount].X := pixel2X(mouseX);
              fpoints[pcount].Y := pixel2Y(mouseY);
              with form1.pointsgrid do
               begin
                cells[1,pcount] := cells[1,1];
                cells[2,pcount] := cells[2,1];
               end;
             end;//if..
             drawmodlines(map2,$000000);
             copymap2box;
           end;
      end;//case
  maDown :
     case mcount of
       0 : begin
            if selpoint <> 0 then
             begin
              mcount := 1;
              drawmodlines(map1,$a0a0a0);
              update1(paintrect);
              copymap12;
              copymap2box;
             end;
           end;
       1,2 : ;
     end;//case
  maUp   :
    case mcount of
      0,1 : begin
             selpoint := 0;
             mcount := 0;
            end;
        2 : begin
             repaintall;
             mcount := 0;
             selpoint := 0;
            end;
    end;//case
 end;//case
 showcursor;
end;

procedure pbedge(pb : TControl; col1,col2 : dword; w : byte);
//draw egde width w around control pb
var x1,y1,x2,y2 : word;
    frm : Tcustomform;
    i : byte;
begin
 frm := getparentform(pb);
 with pb do
  begin
   x1 := left - w;
   y1 := top -w;
   x2 := left + width + w - 1;
   y2 := top + height + w - 1;
  end;
 with frm.canvas do
  begin
   pen.Width := 1;
   for i := 0 to w-1 do
    begin
     pen.Color := col1;
     moveto(x2-i,y1+i);
     lineto(x1+i,y1+i);
     lineto(x1+i,y2-i);
     pen.Color := col2;
     lineto(x2-i,y2-i);
     lineto(x2-i,y1+i);
    end;
  end;//with
end;

//----mouse events----

procedure TForm1.PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var sx,sy : single;
begin
 if (x < 0) or (y < 0) or (x >= boxwidth) or (y >= boxheight) then exit;

 if (mainbutton = mbDraw) or (not (ssShift in shift)) then //draw only in
  if (abs(x-oldX) >= 10) or (abs(y-oldY) >= 10) then       //coarse mode
   begin
    x := ((x+5) div 10) * 10;
    y := ((y+5) div 10) * 10;
   end
    else begin
          x := oldX;
          y := oldY;
         end;

  sx := pixel2X(x);
  xposlabel.Caption := coord2string(sx);
  sy := pixel2Y(y);
  yposlabel.Caption := coord2string(sy);

  mouseX := x;
  mouseY := y;
  mouseAction := maMove;
  case mainbutton of
   mbDraw   : drawProc;
   mbModify : modifyProc;
  end;//case
  oldX := x;
  oldY := y;
end;

procedure TForm1.PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 mouseAction := maDown;
 case mainbutton of
  mbDraw   : drawproc;
  mbModify : modifyProc;
 end;//case
end;

procedure TForm1.PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 mouseAction := maUp;
 case mainbutton of
  mbDraw   : drawproc;
  mbModify : modifyProc;
 end;//case
end;

//---------create , destroy -------------

procedure TForm1.FormCreate(Sender: TObject);
var i : byte;
    s : string;
begin
 with paintbox1 do
  begin
   width := boxwidth;
   height := boxheight;
  end;
 pcount := 0;
 pointsgrid.RowCount := maxpoint+1;
 map1 := Tbitmap.Create;
 with map1 do
  begin
   pixelformat := pf32bit;
   width := boxwidth;
   height := boxheight;
   canvas.brush.Color := $fffffe;
  end;
 map2 := Tbitmap.Create;
 with map2 do
  begin
   pixelformat := pf32bit;
   width := boxwidth;
   height := boxheight;
  end;
 map1full := false;
 map2full := false;
 clearmap1;
 copymap12;
 msgtext.Caption := 'welcome at the polygon expander';
 selpoint := 0;
 mainbutton := mbDraw;
 decimalseparator := '.';
 with pointsgrid do
  begin
   cells[1,0] := 'X';
   cells[2,0] := 'Y';
   colwidths[0] := 20;
   colwidths[1] := 80;
   colwidths[2] := 80;
   for i := 1 to maxpoint do
    begin
     s := inttostr(i);
     if length(s) = 1 then insert(' ',s,1);
     cells[0,i] := s;
    end;
   end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 map1.Free;
 map2.Free;
 map1 := nil;
 map2 := nil;
end;

//---- btn clicks ----------

procedure TForm1.undoBtnClick(Sender: TObject);
begin
 if mainbutton = mbDraw then undo;
 savedpcount := 0;
end;

procedure TForm1.clearbtnClick(Sender: TObject);
//clear button clicked
var i : word;
begin
 msgtext.Caption := '';
 clearpointsGrid;
 for i := 1 to maxpoint do
  with fpoints[i] do
   begin
    x := 0;
    y := 0;
   end;
 pcount := 0;
 savedpcount := 0;
 clearmap1;
 copymap12;
 copymap2box;   
end;

procedure TForm1.drawbtnClick(Sender: TObject);
begin
 mainbutton := mbDraw;
 msgtext.caption := 'draw polygon';
 savedpcount := 0;
end;

procedure TForm1.modifyBtnClick(Sender: TObject);
begin
 mainbutton := mbModify;
 msgtext.caption := 'modify polygon';
 savedpcount := 0;
end;

procedure TForm1.repaintBtnClick(Sender: TObject);
//remove triangles info, background
begin
 repaintall;
end;

//---paints----

procedure TForm1.PaintBox1Paint(Sender: TObject);
var r : Trect;
begin
 r := rect(0,0,boxwidth,boxheight);
 paintbox1.canvas.copyrect(r,map2.canvas,r);
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 pbedge(paintbox1,$0,$c0c0c0,2);
 pbEdge(Hbox,$0,$0,1);
 pbEdge(Vbox,$0,$0,1);
end;

//---general support---

procedure setHBias(b : smallInt);
var r : Trect;
begin
 if b < 40 then b := 40;
 if b > 920 then b := 920;
 b := adjustHVbias(b);
 r := rect(Hbias-8,0,Hbias+8,16);
 with form1.Hbox.Canvas do
  begin
   brush.Color := $ffffff;
   brush.Style := bsSolid;
   fillrect(r);
   pen.Color := $ff0000;
   brush.Color := $ff0000;
   Hbias := b;
   ellipse(Hbias-8,0,Hbias+8,16);
  end;
end;

procedure setVbias(b : smallInt);
var r : Trect;
begin
 if b < 40 then b := 40;
 if b > 760 then b := 760;
 b := adjustHVbias(b);
 r := rect(0,Vbias-8,16,Vbias+8);
 with form1.Vbox.Canvas do
  begin
   brush.Color := $ffffff;
   brush.Style := bsSolid;
   fillrect(r);
   pen.Color := $ff0000;
   brush.Color := $ff0000;
   Vbias := b;
   ellipse(0,Vbias-8,16,Vbias+8);
  end;
end;

procedure clearHVbox(pb : Tpaintbox);
begin
 with pb do with canvas do
  begin
   brush.Color := $ffffff;
   brush.Style := bsSolid;
   fillrect(rect(0,0,width,height));
  end;
end;

procedure delay(msecs : dword);
var t1,t2 : Int64;
    mstime : double;
begin
 t1 := getCPUticks;
 repeat
  application.ProcessMessages;
  t2 := getCPUticks;
  mstime := proctime(t2-t1)*0.001;
 until mstime >= msecs;
end;

procedure procExpansion(EF : boolean);
begin
 while PEF do
  begin
   PolyExpand(EF);
   delay(70); //millisecs
   repaintall;
   showpoints;
  end;
end;

procedure TForm1.HboxPaint(Sender: TObject);
begin
 case TPaintbox(sender).Tag of
  1 : begin
       clearHVbox(Hbox);
       setHbias(Hbias);
      end;
  2 : begin
       clearHVbox(Vbox);
       setVbias(Vbias);
      end;
 end;
end;

procedure TForm1.HboxMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 HVmouseDown := true;
 case TPaintbox(sender).Tag of
  1 : setHbias(adjustHVbias(x));
  2 : setVbias(adjustHVbias(y));
 end;
 repaintall;
 msgtext.Caption := inttostr(Hbias);
end;

procedure TForm1.HboxMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
 if HVmouseDown = false then exit;
 case TPaintbox(sender).Tag of
 1 : setHbias(adjustHVbias(x));
 2 : setVbias(adjustHVbias(y));
 end;
 repaintall;
end;

procedure TForm1.HboxMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 HVmouseDown := false;
end;

procedure TForm1.SpeedButton2Click(Sender: TObject);
var expandflag : boolean;
begin
 expandflag := TSpeedbutton(sender).tag = 2;
 setcanvas(map1.canvas);
 polyExpand(expandflag);
 showpoints;              //in grid
end;

procedure TForm1.SpeedButton1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var ExpandFlag : boolean;
begin
 PEF := true;
 expandflag := TSpeedbutton(sender).tag = 2;
 procExpansion(expandflag);
end;

procedure TForm1.SpeedButton1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 PEF := false;
end;

procedure TForm1.saveBtnClick(Sender: TObject);
var i : byte;
begin
 for i := 1 to pcount do savedpoints[i] := Fpoints[i];
 savedpcount := pcount;
 msgtext.Caption := 'points saved';
 repaintall;
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
var i : byte;
begin
 for i := 1 to savedpcount do fpoints[i] := savedpoints[i];
 pcount := savedpcount;
 msgtext.Caption := 'points reloaded';
 repaintall;
end;

end.
