unit clock_unit;

{
   CPU microseconds clock

   calls:

     procedure setCPUclock;//call once to set proper CPU clock rate
     procedure getCPUticks(var count: Int64);//set count to CPU clock cycles
     function getproctime(t) : double;

   to measure time:

   - record start tick count t1
   - process....
   - record end tick count t2
   - process time = proctime(t2 - t1)
}

interface

function getCPUticks : Int64;
function proctime(t : Int64) : longInt;

implementation

uses windows;

var clockRate : word = 2400;

function getCPUticks : Int64;
asm
 RDTSC;
end;

procedure setCPUclock;
//set variable clockrate
//call this procedure once before measuring any process time
var t1,t2 : cardinal;         //system clock ticks
    cput1,cput2 : Int64;      //CPU clock ticks
begin
 t1 := getTickCount;          //get milliseconds clock
 while getTickCount = t1 do;  //sync with start of 1 millisec interval
 cput1 := GetCPUticks;
 t1 := gettickcount;
 repeat
  cput2 := getCPUticks;
  t2 := gettickcount;
 until t2-t1 >= 500;
 clockrate := trunc((cput2-cput1)/((t2-t1)*1e3));  //set CPU clock in microsecs
end;

function proctime(t : Int64) : longInt;
//convert process ticks t to microsecs
begin
 result := round(t/clockrate);
end;

initialization

 setCPUclock;

end.
